"""
TODO: add me
"""
from __future__ import unicode_literals

import json
import logging

from django.utils.duration import duration_string
from rest_framework import serializers

from eox_core.edxapp_wrapper.courseware import get_courseware_courses
from eox_core.edxapp_wrapper.grades import get_course_grade_factory

from .fields import CustomRelatedField

LOG = logging.getLogger(__name__)


class MetaSerializer(serializers.Serializer):  # pylint: disable=abstract-method
    """Serializer for the blob information in the meta attibute, will return a
    a default dict with empty values if the user does not have the information available
    """
    def to_representation(self, instance):
        # Add all the possible fields
        _fields = {
            'personal_id': None,
        }
        try:
            _data = json.loads(instance)
            _output = {}
            for key, value in iter(_fields.items()):
                _output[key] = _data.get(key, value)
            return _output
        except ValueError:
            if instance:
                LOG.warning("Could not parse metadata during data-api call. %s.", instance)
            return _fields


class UserSerializer(serializers.Serializer):  # pylint: disable=abstract-method
    """
    Serializer for the nested set of variables a user may include
    """
    # ####################################
    # From django.contrib.auth.models.User
    # ####################################

    id = serializers.IntegerField(read_only=True)  # pylint: disable=invalid-name
    username = serializers.CharField(max_length=200, read_only=True)
    first_name = serializers.CharField(max_length=200, read_only=True)
    last_name = serializers.CharField(max_length=200, read_only=True)
    email = serializers.CharField(max_length=200, read_only=True)
    is_active = serializers.BooleanField(read_only=True)
    last_login = serializers.DateTimeField(read_only=True)
    date_joined = serializers.DateTimeField(read_only=True)

    # #################################################
    # From common.djangoapps.student.models.UserProfile
    # #################################################

    name = serializers.CharField(max_length=255, source="profile.name", read_only=True)
    meta = MetaSerializer(source="profile.meta", read_only=True)
    language = serializers.CharField(max_length=255, source="profile.language", read_only=True)
    location = serializers.CharField(max_length=255, source="profile.location", read_only=True)
    year_of_birth = serializers.IntegerField(source="profile.year_of_birth", read_only=True)

    gender = serializers.CharField(source="profile.gender", read_only=True)
    gender_display = serializers.CharField(source="profile.gender_display", read_only=True)

    level_of_education = serializers.CharField(source="profile.level_of_education", read_only=True)
    level_of_education_display = serializers.CharField(source="profile.level_of_education_display", read_only=True)

    mailing_address = serializers.CharField(source="profile.mailing_address", read_only=True)
    city = serializers.CharField(source="profile.city", read_only=True)
    country = serializers.CharField(source="profile.country", read_only=True)
    goals = serializers.CharField(source="profile.goals", read_only=True)
    bio = serializers.CharField(source="profile.bio", max_length=3000, read_only=True)

    # ######################################################
    # From common.djangoapps.student.models.UserSignupSource
    # ######################################################

    site = CustomRelatedField(source='usersignupsource_set', field='site', many=True)


class CourseEnrollmentSerializer(serializers.Serializer):  # pylint: disable=abstract-method
    """
    Serializer for the Course enrollment model
    """
    id = serializers.IntegerField(read_only=True)  # pylint: disable=invalid-name
    user_id = serializers.IntegerField(read_only=True)
    course_id = serializers.CharField(max_length=255, read_only=True)
    created = serializers.DateTimeField(read_only=True)
    is_active = serializers.BooleanField(read_only=True)
    mode = serializers.CharField(max_length=100, read_only=True)


class CourseEnrollmentWithGradesSerializer(CourseEnrollmentSerializer):  # pylint: disable=abstract-method
    """
    Serializer for the course enrollment model, extracting grades
    """
    grades = serializers.SerializerMethodField()

    def get_grades(self, obj):
        """
        TODO: add me
        """
        grade_factory = get_course_grade_factory()
        course = get_courseware_courses().get_course_by_id(obj.course_id)
        user = obj.user
        gradeset = grade_factory().read(user, course).summary
        return gradeset


class CertificateSerializer(serializers.Serializer):  # pylint: disable=abstract-method
    """
    Serializer for Generated Certificates
    """
    # ####################################
    # From certificates.models.GeneratedCertificate
    # ####################################

    username = serializers.CharField(max_length=150, source='user.username', read_only=True)
    name = serializers.CharField(max_length=255, source='user.profile.name', read_only=True)
    course_id = serializers.CharField(max_length=255, read_only=True)
    grade = serializers.FloatField(read_only=True)
    status = serializers.CharField(max_length=100, read_only=True)
    email = serializers.CharField(max_length=200, source='user.email', read_only=True)
    download_url = serializers.CharField(max_length=128, read_only=True)
    verify_uuid = serializers.CharField(max_length=32, read_only=True)
    name_printed_on_certificate = serializers.CharField(max_length=255, source='name', read_only=True)
    mode = serializers.CharField(max_length=32, read_only=True)
    created_date = serializers.DateTimeField(read_only=True)
    modified_date = serializers.DateTimeField(read_only=True)
    key = serializers.CharField(max_length=32, read_only=True)


class ProctoredExamStudentAttemptSerializer(serializers.Serializer):  # pylint: disable=abstract-method
    """
    Serializer for proctored exams attempts made by students
    """
    # ####################################
    # From edx_proctoring.models.ProctoredExamStudentAttempt
    # ####################################

    username = serializers.CharField(max_length=150, source='user.username', read_only=True)
    name = serializers.CharField(max_length=255, source='user.profile.name', read_only=True)
    email = serializers.CharField(max_length=200, source='user.email', read_only=True)
    status = serializers.CharField(max_length=64, read_only=True)
    started_at = serializers.DateTimeField(read_only=True)
    completed_at = serializers.DateTimeField(read_only=True)
    exam_name = serializers.CharField(
        source='proctored_exam.exam_name',
        read_only=True)
    course_id = serializers.CharField(
        max_length=255,
        source='proctored_exam.course_id',
        read_only=True)

    time_taken = serializers.SerializerMethodField()

    def get_time_taken(self, obj):
        """
        TODO: add me
        """
        try:
            time_diff = obj.completed_at - obj.started_at
            result = duration_string(time_diff)
        except TypeError:
            result = None

        return result



"""
TODO: add me
"""
from celery import Task

from eox_core.edxapp_wrapper.users import get_course_enrollment

from .serializers import CourseEnrollmentWithGradesSerializer


class EnrollmentsGrades(Task):
    """
    TODO: add me
    """

    def run(self, data, *args, **kwargs):  # pylint: disable=unused-argument
        """
        This task receives a list with enrollments, and returns the same
        enrollments with grades data
        """

        enrollments_ids = [el["id"] for el in data]
        enrollments_queryset = get_course_enrollment().objects.filter(id__in=enrollments_ids)

        serializer = CourseEnrollmentWithGradesSerializer(enrollments_queryset, many=True)

        return serializer.data



"""Backend for authentication.

This file contains all the necessary authentication dependencies from
https://github.com/eduNEXT/edunext-platform/tree/master/openedx/core/lib/api/authentication.py
"""
from openedx.core.lib.api.authentication import BearerAuthentication  # pylint: disable=import-error


def get_bearer_authentication():
    """Allow to get the function BearerAuthentication from
    https://github.com/eduNEXT/edunext-platform/tree/master/openedx/core/lib/api/authentication.py

    Returns:
        BearerAuthentication function.
    """
    return BearerAuthentication



"""
TODO: add me
"""
from celery.result import AsyncResult
from rest_framework.exceptions import NotFound
from rest_framework.response import Response
from rest_framework.views import APIView


class CeleryTasksStatus(APIView):
    """
    view to check celery tasks status
    """
    authentication_classes = ()

    def get(self, request, task_id=None, *args, **kwargs):  # pylint: disable=unused-argument, keyword-arg-before-vararg
        """
        Return the task status and its result, if already calculated.
        """
        if not task_id:
            raise NotFound()

        task_res = AsyncResult(task_id)
        # Now the existence of the task by id is not validated

        result = None
        if task_res.ready():
            result = task_res.result

        response = {
            "state": task_res.state,
            "result": result,
        }

        return Response(response)



""" Backend test abstraction. """


def get_configuration_helper():
    """ Backend to get the configuration helper. """
    try:
        from openedx.core.djangoapps.site_configuration import \
            helpers as configuration_helpers  # pylint: disable=import-outside-toplevel
    except ImportError:
        configuration_helpers = object
    return configuration_helpers



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Users public function definitions
"""

from importlib import import_module

from django.conf import settings


def get_edxapp_user(*args, **kwargs):
    """ Creates the edxapp user """

    backend_function = settings.EOX_CORE_USERS_BACKEND
    backend = import_module(backend_function)

    return backend.get_edxapp_user(*args, **kwargs)


def create_edxapp_user(*args, **kwargs):
    """ Creates the edxapp user """

    backend_function = settings.EOX_CORE_USERS_BACKEND
    backend = import_module(backend_function)

    return backend.create_edxapp_user(*args, **kwargs)


def delete_edxapp_user(*args, **kwargs):
    """ Deletes the edxapp user """

    backend_function = settings.EOX_CORE_USERS_BACKEND
    backend = import_module(backend_function)

    return backend.delete_edxapp_user(*args, **kwargs)


def get_user_read_only_serializer(*args, **kwargs):
    """ Gets the Open edX model UserProfile """

    backend_function = settings.EOX_CORE_USERS_BACKEND
    backend = import_module(backend_function)

    return backend.get_user_read_only_serializer(*args, **kwargs)


def check_edxapp_account_conflicts(*args, **kwargs):
    """ Checks the db for accounts with the same email or password """

    backend_function = settings.EOX_CORE_USERS_BACKEND
    backend = import_module(backend_function)

    return backend.check_edxapp_account_conflicts(*args, **kwargs)


def get_course_enrollment():
    """ Gets the CourseEnrollment model """

    backend_function = settings.EOX_CORE_USERS_BACKEND
    backend = import_module(backend_function)

    return backend.get_course_enrollment()


def get_course_team_user(*args, **kwargs):
    """ Gets the course_team_user function """

    backend_function = settings.EOX_CORE_USERS_BACKEND
    backend = import_module(backend_function)

    return backend.get_course_team_user(*args, **kwargs)


def get_user_signup_source():
    """ Gets the UserSignupSource model """

    backend_function = settings.EOX_CORE_USERS_BACKEND
    backend = import_module(backend_function)

    return backend.get_user_signup_source()


def get_user_profile():
    """ Gets the UserProfile model """

    backend_function = settings.EOX_CORE_USERS_BACKEND
    backend = import_module(backend_function)

    return backend.get_user_profile()


def get_username_max_length():
    """ Gets max length allowed for the username"""
    backend_function = settings.EOX_CORE_USERS_BACKEND
    backend = import_module(backend_function)
    return backend.USERNAME_MAX_LENGTH


def generate_password(*args, **kwargs):
    """
    Runs the generate_password funcion of edx-platform used to generate
     a random password.
    """
    backend_function = settings.EOX_CORE_USERS_BACKEND
    backend = import_module(backend_function)
    return backend.generate_password(*args, **kwargs)


def get_user_attribute():
    """ Gets the UserAttribute model """
    backend_function = settings.EOX_CORE_USERS_BACKEND
    backend = import_module(backend_function)

    return backend.get_user_attribute()



"""
Backend for certificates app.
"""

from lms.djangoapps.certificates.api import get_certificate_url as _get_certificate_url  # pylint: disable=import-error
from lms.djangoapps.certificates.models import GeneratedCertificate  # pylint: disable=import-error
from lms.djangoapps.certificates.tasks import generate_certificate  # pylint: disable=import-error


def get_generated_certificate():
    """Get GeneratedCertificate model."""
    return GeneratedCertificate


def get_generate_certificate_task(**kwargs):
    """Get generate_certificate_task task."""
    return generate_certificate(**kwargs)


def get_certificate_url(**kwargs):
    """Get get_certificate_url function."""
    return _get_certificate_url(**kwargs)



"""
Test module for users viewset.
"""
from django.contrib.auth.models import User
from django.test import TestCase, override_settings
from django.urls import reverse
from mock import MagicMock, patch
from rest_framework import status
from rest_framework.serializers import ValidationError
from rest_framework.test import APIClient

from eox_core.api.v1.serializers import EdxappUserQuerySerializer, WrittableEdxappUserSerializer

EXTENDED_PROFILE_FIELDS = [
    "father_last_name",
    "mother_last_name",
    "type_document",
    "personal_id",
    "mobile",
    "address",
]

EDNX_CUSTOM_REGISTRATION_FIELDS = [
    {
        "label": "Father last name",
        "name": "father_last_name",
        "type": "text",
    },
    {
        "label": "Mother last name",
        "name": "mother_last_name",
        "type": "text",
    },
    {
        "label": "Document type",
        "name": "type_document",
        "options": [
            "dni",
            "passport",
            "license",
        ],
        "type": "select",
        "default": "dni",
    },
    {
        "label": "Document number",
        "name": "personal_id",
        "type": "text",
    },
    {
        "label": "Mobile",
        "name": "mobile",
        "type": "text",
        "restrictions": {
            "max_length": 12,
        }
    },
    {
        "label": "Address",
        "name": "address",
        "type": "text",
    },
]

REGISTRATION_EXTRA_FIELDS = {
    "address": "required",
    "country": "required",
    "father_last_name": "required",
    "goals": "required",
    "mailing_address": "hidden",
    "mobile": "required",
    "mother_last_name": "required",
    "personal_id": "required",
    "type_document": "required",
}


@override_settings(
    extended_profile_fields=EXTENDED_PROFILE_FIELDS,
    EDNX_CUSTOM_REGISTRATION_FIELDS=EDNX_CUSTOM_REGISTRATION_FIELDS,
    REGISTRATION_EXTRA_FIELDS=REGISTRATION_EXTRA_FIELDS,
    EOX_CORE_USER_UPDATE_SAFE_FIELDS=[
        "is_active",
        "password",
        "fullname",
        "mailing_address",
        "year_of_birth",
        "gender",
        "level_of_education",
        "city",
        "country",
        "goals",
        "bio",
        "phone_number",
    ],
)
class UsersUpdaterAPITest(TestCase):
    """Test class for EdxappUserUpdater viewset."""

    patch_permissions = patch('eox_core.api.v1.permissions.EoxCoreAPIPermission.has_permission', return_value=True)

    def setUp(self):
        """Setup method for test class."""
        self.user = User(username="test", email="test@example.com", password="testtest")
        self.user.profile = MagicMock()
        self.client = APIClient()
        self.url = reverse("eox-api:eox-api:edxapp-user-updater")
        self.client.force_authenticate(user=self.user)

    @patch_permissions
    @patch('eox_core.api.v1.serializers.UserSignupSource')
    @patch('eox_core.api.v1.views.get_edxapp_user')
    @patch('eox_core.api.v1.views.EdxappUserReadOnlySerializer')
    def test_update_success(self, user_serializer, get_edxapp_user, signup_source, _):
        """Used to test updating safe fields of an edxapp user."""
        update_data = {
            "email": "test",
            "fullname": "test",
        }
        user_serializer.return_value.data = {}
        get_edxapp_user.return_value = self.user
        signup_source.objects.filter.return_value.count.return_value = 1

        response = self.client.patch(self.url, data=update_data, format="json")

        self.assertEqual(status.HTTP_200_OK, response.status_code)

    @patch_permissions
    @patch('eox_core.api.v1.serializers.UserSignupSource')
    @patch('eox_core.api.v1.views.get_edxapp_user')
    def test_update_bad_sign_up_source(self, get_edxapp_user, signup_source, _):
        """Used to test that when an user has more than one signup source then the update can't be done."""
        update_data = {
            "email": "test",
            "fullname": "test",
        }
        get_edxapp_user.return_value = self.user
        signup_source.objects.filter.return_value.count.return_value = 2

        response = self.client.patch(self.url, data=update_data, format="json")

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual(response.content, '{"detail":["You can\'t update users with more than one sign up source."]}'
                         .encode())

    @patch_permissions
    @patch('eox_core.api.v1.serializers.UserSignupSource')
    @patch('eox_core.api.v1.views.get_edxapp_user')
    def test_update_role_not_allowed(self, get_edxapp_user, signup_source, _):
        """Used to test that if an user is staff or superuser then the update can't be done."""
        user = User(username="test", email="test@example.com", password="testtest", is_staff=True)
        update_data = {
            "email": "test",
            "fullname": "test",
        }
        get_edxapp_user.return_value = user
        signup_source.objects.filter.return_value.count.return_value = 1

        response = self.client.patch(self.url, data=update_data, format="json")

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual(response.content, '{"detail":["You can\'t update users with roles like staff or superuser."]}'
                         .encode())

    @patch_permissions
    @patch('eox_core.api.v1.serializers.UserSignupSource')
    @patch('eox_core.api.v1.views.get_edxapp_user')
    @patch('eox_core.api.v1.views.EdxappUserReadOnlySerializer')
    def test_update_extra_registration_fields_success(self, user_serializer, get_edxapp_user, signup_source, _):
        """Used to test updating safe extra fields of an edxapp user."""
        update_data = {
            "email": "test",
            "fullname": "test",
            "country": "gr",
            "goals": "new goals",
        }
        user_serializer.return_value.data = {}
        get_edxapp_user.return_value = self.user
        signup_source.objects.filter.return_value.count.return_value = 1

        response = self.client.patch(self.url, data=update_data, format="json")

        self.assertEqual(status.HTTP_200_OK, response.status_code)

    @override_settings(EOX_CORE_USER_UPDATE_SAFE_FIELDS=["is_active", "password", "fullname"])
    @patch_permissions
    @patch('eox_core.api.v1.serializers.UserSignupSource')
    @patch('eox_core.api.v1.views.get_edxapp_user')
    @patch('eox_core.api.v1.views.EdxappUserReadOnlySerializer')
    def test_update_unsafe_user_profile_fields(self, user_serializer, get_edxapp_user, signup_source, _):
        """If a non safe UserProfile field wants to be updated, then the request must return a
        400_BAD_REQUEST status code with the proper validation error message.
        """
        update_data = {
            "email": "test",
            "fullname": "test",
            "country": "gr",
        }
        user_serializer.return_value.data = {}
        get_edxapp_user.return_value = self.user
        signup_source.objects.filter.return_value.count.return_value = 1

        response = self.client.patch(self.url, data=update_data, format="json")

        self.assertRaises(ValidationError)
        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual(response.content, '{"detail":["You are not allowed to update country."]}'
                         .encode())

    @patch_permissions
    @patch('eox_core.api.v1.serializers.UserSignupSource')
    @patch('eox_core.api.v1.views.get_edxapp_user')
    @patch('eox_core.api.v1.views.EdxappUserReadOnlySerializer')
    def test_fail_to_update_non_configured_extra_fields(self, user_serializer, get_edxapp_user, signup_source, _):
        """Tests the WrittableEdxappUserSerializer. A field that has not been properly added
        to the registration extra fields settings can't be updated.
        """
        update_data = {
            "email": "test@example.com",
            "fullname": "test",
            "country": "ve",
            "year_of_birth": 1997,
            "favorite_food": "pizza",
            "level_of_education": "m",
        }
        validated_data = {
            "email": "test@example.com",
            "fullname": "test",
            "country": "ve",
        }
        user_serializer.return_value.data = {}
        get_edxapp_user.return_value = self.user
        signup_source.objects.filter.return_value.count.return_value = 1

        serializer = WrittableEdxappUserSerializer(self.user, data=update_data, partial=True)
        serializer.is_valid()

        self.assertEqual(validated_data, serializer.data)

    @override_settings(REGISTRATION_EXTRA_FIELDS={
        "address": "required",
        "country": "required",
        "father_last_name": "required",
        "goals": "required",
        "mailing_address": "hidden",
        "mobile": "required",
        "mother_last_name": "required",
        "personal_id": "required",
        "type_document": "required",
        "year_of_birth": "hidden",
        "level_of_education": "required",
    })
    @patch_permissions
    @patch('eox_core.api.v1.serializers.UserSignupSource')
    @patch('eox_core.api.v1.views.get_edxapp_user')
    @patch('eox_core.api.v1.views.EdxappUserReadOnlySerializer')
    def test_success_update_configured_extra_fields(self, user_serializer, get_edxapp_user, signup_source, _):
        """Tests the WrittableEdxappUserSerializer. A field that has been properly configured
        as a extra registration field, can be successfully updated.
        """
        update_data = {
            "email": "test@example.com",
            "country": "ve",
            "level_of_education": "m",
        }
        user_serializer.return_value.data = {}
        get_edxapp_user.return_value = self.user
        signup_source.objects.filter.return_value.count.return_value = 1

        serializer = WrittableEdxappUserSerializer(self.user, data=update_data, partial=True)
        serializer.is_valid()

        self.assertEqual(update_data, serializer.data)

    @patch_permissions
    @patch('eox_core.api.v1.serializers.UserSignupSource')
    @patch('eox_core.api.v1.views.get_edxapp_user')
    @patch('eox_core.api.v1.views.EdxappUserReadOnlySerializer')
    def test_update_hidden_field(self, user_serializer, get_edxapp_user, signup_source, _):
        """Tests the WrittableEdxappUserSerializer when the request tries to update a
        field that is set as "hidden" in the REGISTRATION_EXTRA_FIELD setting.
        """
        update_data = {
            "email": "test@example.com",
            "country": "ve",
            "year_of_birth": 1997,
            "mailing_address": "test@example.com",
        }
        validated_data = {
            "email": "test@example.com",
            "country": "ve",
        }
        user_serializer.return_value.data = {}
        get_edxapp_user.return_value = self.user
        signup_source.objects.filter.return_value.count.return_value = 1

        serializer = WrittableEdxappUserSerializer(self.user, data=update_data, partial=True)
        serializer.is_valid()

        self.assertEqual(validated_data, serializer.data)

    @patch_permissions
    @patch('eox_core.api.v1.serializers.UserSignupSource')
    @patch('eox_core.api.v1.views.get_edxapp_user')
    @patch('eox_core.api.v1.views.EdxappUserReadOnlySerializer')
    def test_restriction_in_text_field(self, user_serializer, get_edxapp_user, signup_source, _):
        """tests that the validations of a custom field of type 'text'
        are being made when updating a User. In this case the mobile field
        exceeds the max_length defined in the settings."""
        update_data = {
            "email": "test@example.com",
            "mobile": "090017288812344",
        }
        user_serializer.return_value.data = {}
        get_edxapp_user.return_value = self.user
        signup_source.objects.filter.return_value.count.return_value = 1

        response = self.client.patch(self.url, data=update_data, format="json")

        self.assertRaises(ValidationError)
        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual(response.content, '{"mobile":["Ensure this field has no more than 12 characters."]}'
                         .encode())

    @patch_permissions
    @patch('eox_core.api.v1.serializers.UserSignupSource')
    @patch('eox_core.api.v1.views.get_edxapp_user')
    @patch('eox_core.api.v1.views.EdxappUserReadOnlySerializer')
    def test_wrong_option_in_select_field(self, user_serializer, get_edxapp_user, signup_source, _):
        """tests the case when the request data has an invalid option in a custom select field.
        For this case the updating process is not successful.
        """
        update_data = {
            "email": "test@example.com",
            "type_document": "credit card",
        }
        user_serializer.return_value.data = {}
        get_edxapp_user.return_value = self.user
        signup_source.objects.filter.return_value.count.return_value = 1

        response = self.client.patch(self.url, data=update_data, format="json")

        self.assertRaises(ValidationError)
        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual(response.content, '{"type_document":["\\"credit card\\" is not a valid choice."]}'
                         .encode())

    @patch_permissions
    @patch('eox_core.api.v1.serializers.UserSignupSource')
    @patch('eox_core.api.v1.views.get_edxapp_user')
    @patch('eox_core.api.v1.views.EdxappUserReadOnlySerializer')
    def test_required_field(self, user_serializer, get_edxapp_user, signup_source, _):
        """tests the case when the request tries to set a required field as None.
        The requests returns a 400_BAD_REQUEST status code and shows an error message.
        """
        update_data = {
            "email": "test@example.com",
            "country": None,
        }
        user_serializer.return_value.data = {}
        get_edxapp_user.return_value = self.user
        signup_source.objects.filter.return_value.count.return_value = 1

        response = self.client.patch(self.url, data=update_data, format="json")

        self.assertRaises(ValidationError)
        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual(response.content, '{"country":["This field may not be null."]}'
                         .encode())


@override_settings(
    extended_profile_fields=EXTENDED_PROFILE_FIELDS,
    EDNX_CUSTOM_REGISTRATION_FIELDS=EDNX_CUSTOM_REGISTRATION_FIELDS,
    REGISTRATION_EXTRA_FIELDS=REGISTRATION_EXTRA_FIELDS,
)
class UsersAPITest(TestCase):
    """Test class for users viewset."""

    patch_permissions = patch('eox_core.api.v1.permissions.EoxCoreAPIPermission.has_permission', return_value=True)

    def setUp(self):
        """Setup method for test class."""
        self.gender_choices = (
            ('m', 'Male'),
            ('f', 'Female'),
            ('o', 'Other/Prefer Not to Say')
        )
        self.level_of_education_choices = (
            ('p', 'Doctorate'),
            ('m', 'Master or professional degree'),
            ('el', 'Elementary/primary school'),
            ('none', 'No formal education'),
            ('other', 'Other education')
        )
        self.user = User(username="test", email="test@example.com", password="testtest")
        self.client = APIClient()
        self.url = reverse("eox-api:eox-api:edxapp-user")
        self.client.force_authenticate(user=self.user)

    @patch_permissions
    @patch('eox_core.api.v1.serializers.check_edxapp_account_conflicts')
    @patch('eox_core.api.v1.serializers.get_gender_choices')
    @patch('eox_core.api.v1.serializers.get_level_of_education_choices')
    def test_success(self, mock_level_education_choices, mock_gender_choices, mock_check_account_conflicts, _):
        """test when the request data is correct and complete
        the serializer is valid.
        """
        request_data = {
            "username": "AleMagno",
            "email": "alexTheGreat@example.com",
            "password": "p@ssword",
            "fullname": "Alexander",
            "mother_last_name": "Perez",
            "type_document": "dni",
            "father_last_name": "Magno",
            "personal_id": "0900172",
            "mobile": "04143472044",
            "address": "Pella",
            "country": "gr",
            "goals": "to have all the people in the places I have conquered accept me as their ruler.",
        }
        mock_level_education_choices.return_value = self.level_of_education_choices
        mock_gender_choices.return_value = self.gender_choices
        mock_check_account_conflicts.return_value = False

        serializer = EdxappUserQuerySerializer(data=request_data)

        self.assertTrue(serializer.is_valid())
        self.assertEqual(set(serializer.errors), set([]))

    @patch_permissions
    @patch('eox_core.api.v1.serializers.check_edxapp_account_conflicts')
    @patch('eox_core.api.v1.serializers.get_gender_choices')
    @patch('eox_core.api.v1.serializers.get_level_of_education_choices')
    @patch('eox_core.api.v1.views.create_edxapp_user')
    def test_missing_required_fields(self, create_edxapp_user, mock_level_education_choices, mock_gender_choices, mock_check_account_conflicts, _):
        """tests case when the request data has missing required fields,
        the registration process is not successful.
        """
        request_data = {
            "username": "AleMagno",
            "email": "alexTheGreat@example.com",
            "password": "p@ssword",
            "fullname": "Alexander",
            "mother_last_name": "Perez",
            "type_document": "dni",
            "father_last_name": "Magno",
            "personal_id": "0900172",
            "mobile": "04143472044",
            "address": "Pella",
            "goals": "to have all the people in the places I have conquered accept me as their ruler.",
        }
        mock_level_education_choices.return_value = self.level_of_education_choices
        mock_gender_choices.return_value = self.gender_choices
        mock_check_account_conflicts.return_value = False

        response = self.client.post(self.url, data=request_data, format="json")

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual(response.content, '{"country":["This field is required."]}'
                         .encode())

    @patch_permissions
    @patch('eox_core.api.v1.serializers.check_edxapp_account_conflicts')
    @patch('eox_core.api.v1.serializers.get_gender_choices')
    @patch('eox_core.api.v1.serializers.get_level_of_education_choices')
    @patch('eox_core.api.v1.views.create_edxapp_user')
    def test_wrong_option_in_select_field(self, create_edxapp_user, mock_level_education_choices, mock_gender_choices, mock_check_account_conflicts, _):
        """tests case when the request data has an invalid option in a custom select field,
        the registration process is not successful.
        """
        request_data = {
            "username": "AleMagno",
            "email": "alexTheGreat@example.com",
            "password": "p@ssword",
            "fullname": "Alexander",
            "mother_last_name": "Perez",
            "type_document": "credit card",
            "father_last_name": "Magno",
            "personal_id": "0900172",
            "mobile": "04143472044",
            "address": "Pella",
            "country": "gr",
            "goals": "to have all the people in the places I have conquered accept me as their ruler.",
        }
        mock_level_education_choices.return_value = self.level_of_education_choices
        mock_gender_choices.return_value = self.gender_choices
        mock_check_account_conflicts.return_value = False

        response = self.client.post(self.url, data=request_data, format="json")

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual(response.content, '{"type_document":["\\"credit card\\" is not a valid choice."]}'
                         .encode())

    @patch_permissions
    @patch('eox_core.api.v1.serializers.check_edxapp_account_conflicts')
    @patch('eox_core.api.v1.serializers.get_gender_choices')
    @patch('eox_core.api.v1.serializers.get_level_of_education_choices')
    def test_default_in_select_field(self, mock_level_education_choices, mock_gender_choices, mock_check_account_conflicts, _):
        """tests the case when a custom field of type select is required
        and has a default value, the serializer is valid.
        """
        request_data = {
            "username": "AleMagno",
            "email": "alexTheGreat@example.com",
            "password": "p@ssword",
            "fullname": "Alexander",
            "mother_last_name": "Perez",
            "father_last_name": "Magno",
            "personal_id": "0900172",
            "mobile": "04143472044",
            "address": "Pella",
            "country": "gr",
            "goals": "to have all the people in the places I have conquered accept me as their ruler.",
        }
        mock_level_education_choices.return_value = self.level_of_education_choices
        mock_gender_choices.return_value = self.gender_choices
        mock_check_account_conflicts.return_value = False

        serializer = EdxappUserQuerySerializer(data=request_data)

        self.assertTrue(serializer.is_valid())
        self.assertEqual(set(serializer.errors), set([]))

    @patch_permissions
    @patch('eox_core.api.v1.serializers.check_edxapp_account_conflicts')
    @patch('eox_core.api.v1.serializers.get_gender_choices')
    @patch('eox_core.api.v1.serializers.get_level_of_education_choices')
    @patch('eox_core.api.v1.views.create_edxapp_user')
    def test_restriction_in_text_field(self, create_edxapp_user, mock_level_education_choices, mock_gender_choices, mock_check_account_conflicts, _):
        """tests that the validations of a custom field of type 'text'
        are being made. In this case the mobile field exceeds the max_length
        defined in the settings.
        """
        request_data = {
            "username": "AleMagno",
            "email": "alexTheGreat@example.com",
            "password": "p@ssword",
            "fullname": "Alexander",
            "mother_last_name": "Perez",
            "type_document": "dni",
            "gender": "m",
            "father_last_name": "Magno",
            "personal_id": "0900172",
            "mobile": "0414347204400999",
            "address": "Pella",
            "country": "gr",
            "goals": "graduate",
        }
        mock_level_education_choices.return_value = self.level_of_education_choices
        mock_gender_choices.return_value = self.gender_choices
        mock_check_account_conflicts.return_value = False

        response = self.client.post(self.url, data=request_data, format="json")

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual(response.content, '{"mobile":["Ensure this field has no more than 12 characters."]}'
                         .encode())
